<? if ( !defined("_INCLUDED_GLOBAL") ) die("Access Denied");		// [Security]Allow_url_fopen = On ?>
<?

/**
 * [2008-02-19] 대용량첨부 다운횟수/기간 초과처리
 *   - 대용량첨부 만료 및 파일 삭제 위치
 *     - 대용량첨부 다운로드시 만료 및 파일 삭제 처리
 *     - 자신의 '대용량 첨부내역' 접근시, 목록에서 보여지기전 검사후 만료 및 파일 삭제 처리
 *     - 하루 1회(디비 동기화/최초 메일 수신시) 이미 만료일이 지난 대용량 첨부의 파일을 삭제 처리
 *
 * @param array $bigfile
 * @param bool|null $is_expire 함수 실행전 이미 만료로 검사된 경우, 함수 안에서는 만료 검사를 생략하고 바로 만료 처리
 * @param string|null $upload_dir 배치 자동 실행시 사용하루 1회 자동 실행되는 배치 CLI 모드에서 사용(디비 동기화(혹은 최초 메일 수신시) CLI 모드용.  로그인 회원이 아닌, 입력된 업로드 주소로 처리.
 *
 * @return bool
 * @throws Exception
 */
function bigfile_expire($bigfile, $is_expire=true, $upload_dir='')
{
	global $TB, $G_SYS, $UPLOAD_ROOT, $UPLOAD_ROOT_NEW;

	if ($bigfile['bf_expire'] != 'Y')
	{
		// 커스텀) 특정 회원만 대용량 첨부 만료 처리를 생략할 경우 이곳에서 선언해주세요.
		//  주의) 소스 수정후에는 반드시 다운 횟수 및 기간 초과시 삭제되지 않고 유지되는지 테스트를 통한 검증이 필요합니다.
		//   ex)   if ($bigfile['m_no'] == '1') { return false; }     // 회원번호가 1번일 경우 대용량 첨부 만료 처리 생략하고 영구 보관

		// [2023-12-22] 대용량 첨부 만료 제외 회원번호 (대용량 첨부 영구 보관용, 단 보낸 사람은 삭제 가능)
		if (is_array($G_SYS['NM_BIGFILE_EXPIRE_EXCLUDE_NO']) && count($G_SYS['NM_BIGFILE_EXPIRE_EXCLUDE_NO']) > 0)
		{
			if (in_array($bigfile['m_no'], $G_SYS['NM_BIGFILE_EXPIRE_EXCLUDE_NO']))
			{
				errorlog("[bigfile_expire] 만료 처리 생략 - 대용량 첨부 만료 제외 회원번호(\$G_SYS['NM_BIGFILE_EXPIRE_EXCLUDE_NO']) | " . json_encode($bigfile), 'DEBUG');

				return false;
			}
		}

		if ($upload_dir == '')
		{
			$q = "SELECT m_id, m_mail_file_group, d_no FROM ".$TB['MEMBER']." WHERE m_no='".$bigfile['m_no']."'";
			$mbr = db_get_row($q);
			$upload_dir = 'bigfile/'.get_d_name($mbr['d_no']).'/'.$mbr['m_mail_file_group'].'/'.$mbr['m_id'];
		}	// if()
		$UPLOAD_ROOT_ORI = $UPLOAD_ROOT;
		$UPLOAD_ROOT = $UPLOAD_ROOT_NEW;

		// 대용량 첨부파일 다운로드시 기간 및 횟수 검사를 하지 않은 경우에만 검사 진행
		if (!$is_expire)
		{
			// 만료여부 검사
            // [2021-11] php 32bit timestamp 계산 오류개선
			#$limit_date = date('Y-m-d', (strtotime($bigfile['bf_regdate'])+(86400*($G_SYS[UPLOAD_MAIL_BIGFILE_DAY]-1))));
            $date = new DateTime($bigfile['bf_regdate']);
            $date->add(new DateInterval('P'.$G_SYS[UPLOAD_MAIL_BIGFILE_DAY].'D'));
            $limit_date = $date->format('Y-m-d') . "\n";

			if ($bigfile['bf_down'] >= $G_SYS[UPLOAD_MAIL_BIGFILE_COUNT])
			{
				$is_expire = true;
				errorlog("[bigfile_expire] 다운로드 가능횟수 초과로 만료 및 파일 삭제 | " . $G_SYS[UPLOAD_MAIL_BIGFILE_COUNT] . " | " . json_encode($bigfile), 'DEBUG');
			} else if (date('Y-m-d') > $limit_date)
			{
				$is_expire = true;
				errorlog("[bigfile_expire] 다운로드 가능기간 초과로 만료 및 파일 삭제 | " . $limit_date . " | " . json_encode($bigfile), 'DEBUG');
			} else if (!file_exists($UPLOAD_ROOT.'/'.$upload_dir.'/'.$bigfile['bf_key']))		// [2014-05-28] 파일이 존재하지 않을 경우, 만료로 처리.
			{
				// NAS 스토리지 등의 일시적인 장애 가능성을 고려하여 파일 없을 경우 만료 처리 생략
				errorlog("[bigfile_expire] 서버 파일이 알 수없는 원인으로 존재하지 않음(만료 처리 생략) | " . $UPLOAD_ROOT.'/'.$upload_dir.'/'.$bigfile['bf_key'] . " | " . json_encode($bigfile), 'DEBUG');

				return false;
			} else
			{
				return false;
			}	// if()
		}	// if()

		$sql = "
			UPDATE
				".$TB['BIGFILE']."
			SET
				bf_expire='Y'
			WHERE
				bf_key='" . $bigfile['bf_key'] . "'
				AND m_no='" . $bigfile['m_no'] . "'
			";
		db_query($sql);

		if ($bigfile['bf_type'] == 'B')
		{
			// file del - 대용량 첨부, 다운로드 횟수/기간이 초과하여 파일이 삭제된 경우를 고려하여 파일이 존재할때만 삭제함.
				if (file_exists($UPLOAD_ROOT.'/'.$upload_dir.'/'.$bigfile['bf_key']))
				{
					require_once("include/file.php");

					// [2023-12-22] 대용량 첨부 만료후에도 파일 삭제 생략 (대용량 첨부 영구 보관용, 단 보낸 사람은 삭제 가능)
					if (!$G_SYS['NM_BIGFILE_EXPIRE_SKIP_FILE_DEL']) {
						$errorMsg = file_del($bigfile['bf_key'], '', $upload_dir);
						if ($errorMsg) {
							errorlog(
								"대용량 첨부 파일 삭제가 실패하였습니다.\n=> " . $UPLOAD_ROOT . '/' . $upload_dir . '/'
								. $bigfile['bf_key'] . $errorMsg, 'FILE'
							);
						}
					}

					require_once('include/member_size_lib.php');
					member_size_update($bigfile['m_no'], 'BIGFILE', -($bigfile['bf_size']));
				}	// if()
		}	// if()

		$UPLOAD_ROOT = $UPLOAD_ROOT_ORI;
	} else
	{
		$is_expire = true;
	}	// if()

	return $is_expire;
}	// function()


// [2008-02-19] 임시 업로드 디렉토리에서 3시간이 지난 파일은 자동 삭제.
// bigfile_upload_tmp_del($UPLOAD_DIR[BIGFILE_TMP]);
function bigfile_upload_tmp_del($upload_dir)
{
	global $G_SYS, $UPLOAD_ROOT, $UPLOAD_ROOT_NEW;

	$G_SYS['BIGFILE_UPLOAD_TMP_DEL_HOUR'];

	$UPLOAD_ROOT_ORI = $UPLOAD_ROOT;
	$UPLOAD_ROOT = $UPLOAD_ROOT_NEW;

	$tmpdir = $UPLOAD_ROOT_NEW.'/'.$upload_dir;

	// 삭제 기준 시간
	if ($G_SYS['BIGFILE_UPLOAD_TMP_DEL_HOUR'])
		$G_SYS['BIGFILE_UPLOAD_TMP_DEL_HOUR'] = 3;

	$del_time = (time() - ($G_SYS['BIGFILE_UPLOAD_TMP_DEL_HOUR']*3600));

	if (is_dir($tmpdir))
	{
		// [2008-06-04] 편지함 복구시 임시 디렉토리 삭제
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
		{
			$cmd_rm = 'rmdir/q/s';
		} else
		{
			$cmd_rm = 'rm -rf';
		}	// if()

		// [2009-06-09] 임시 디렉토리 삭제시 오류 해결
		require_once("include/file.php");

		$files = array();
		if ($dh = opendir($tmpdir)) {
			while (($file = readdir($dh)) !== false) {
				if ($file!="." && $file!=".." && is_file($tmpdir.'/'.$file) && filemtime($tmpdir.'/'.$file) <= $del_time)
				{
					// [2022-09-29] Windows 에서 읽어온 파일명은 CP949 이므로, UTF-8 로 변환하여 삭제 함수 호출해야 함
					if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN' && strtoupper($G_SYS[CHARSET]) == 'UTF-8') {
						$file = iconv_fix('CP949', $file, 'UTF-8');
					}

					$errorMsg = file_del($file, "", $upload_dir);
					if ($errorMsg)
						errorlog("임시 업로드 파일 삭제가 실패하였습니다.\n=> " . $tmpdir. '/' . $file . $errorMsg, 'FILE');
				} else if (strpos($file, 'NmailRestore_') !== false && is_dir($tmpdir.'/'.$file) && filemtime($tmpdir.'/'.$file) <= $del_time)
				{
					// [2008-06-04] 편지함 복구시 임시 디렉토리 삭제
					if ($tmpdir != '' && strpos($file, 'NmailRestore_') !== false)		// 프로그램상의 미스로 인하여 / 파티션이 삭제되는 것 예방.
						system($cmd_rm.' '.escapeshellarg(realpath($tmpdir.'/'.$file.'/')));
				}	// if()
			}
			closedir($dh);
		} else
		{
			errorlog("임시 업로드 디렉토리의 퍼미션이 잘못되어 자동 삭제가 실패하였습니다.\n=> " . $tmpdir, 'FILE');
		}	// if()
	}	// if()

	$UPLOAD_ROOT = $UPLOAD_ROOT_ORI;
}	// function()


?>