<?php
if ( ! defined("_INCLUDED_GLOBAL")) {
	die("Access Denied");
}
/**
 * HTML purifier
 *   - 메일 본문 등에서 XSS 방어
 */

/**
 * HTML purifier 설정
 *   - 설정 가이드) http://htmlpurifier.org/live/configdoc/plain.html
 *
 * @return array
 */
function purifier_config()
{
	return array(
		'Core.Encoding'         => 'UTF-8',
		'Cache.DefinitionImpl'  => null,     // 개별 메일 본문 위주이므로 캐시 비활성화
		'HTML.Doctype'          => 'HTML 4.01 Transitional',
		'HTML.Allowed'          => 'div[style],b[style],strong[style],small[style],i[style],em[style],u[style],a[href|title|target|style],ul[style],ol[style],li[style],p[style],br[style],span[style],font[style],img[width|height|alt|src|style],'
			. 'h1[style],h2[style],h3[style],h4[style],h5[style],h6[style],'
			. 'pre[style],address[style],'
			//. 'aside[style],footer[style],header[style],main[style],nav[style],section[style],'
			. 'blockquote[style],dd[style],dl[style],dt[style],hr[style],menu[style],'
//			. 'area[style],map[style],'
			. 'del[style],ins[style],'
			. 'table[width|border|cellpadding|cellspacing|align|style|summary],caption[style],colgroup,col[width],thead,tbody,tr[style],th[style|width|height|colspan|rowspan|align|valign|bgcolor],td[style|width|height|colspan|rowspan|align|valign|bgcolor]',
		'CSS.AllowedProperties' => 'font,font-size,font-weight,font-style,font-family,text-decoration,'
			. 'color,background-color,text-align,letter-spacing,'
			. 'width,max-width,height,line-height,'
			. 'padding,padding-top,padding-bottom,padding-left,padding-right,'
			. 'margin,margin-top,margin-bottom,margin-left,margin-right,'
			. 'border,border-top,border-bottom,border-left,border-right,'
			. 'border-width,border-top-width,border-bottom-width,border-left-width,border-right-width,'
			. 'border-style,border-top-style,border-bottom-style,border-left-style,border-right-style,'
			. 'border-color,border-top-color,border-bottom-color,border-left-color,border-right-color,'
			. 'border-collapse,'
			. 'list-style,'
			. 'background,vertical-align,display,overflow,float',
		'CSS.AllowTricky'       => true,      // display: none 등 허용
		'Attr.DefaultImageAlt'  => '',       // img alt 속성이 없을때 이미지 주소를 기본으로 넣지 않도록 빈 문자열 선언
		'URI.Host'              => $_SERVER['HTTP_HOST'],
		'HTML.TargetBlank'      => true,     // 외부 링크는 모두 새창
		'URI.AllowedSchemes'    => array(
			'http'   => true,
			'https'  => true,
			'mailto' => true,
			'ftp'    => true,
			'nntp'   => true,
			'news'   => true,
			'tel'    => true,
			'data'   => true,     // <img src="data:image/png;base64 지원
		),
	);
}

/**
 * HTML 안전한 문자열만 유지
 *
 * @param string $content
 *
 * @return string
 * @throws Exception
 */
function purifier_clean($content)
{
	global $G_SYS;

	// HTML 원본 보기
	if (isset($_GET['ov']) && $_GET['ov']) {
		return $content;
	}

	if ( ! isset($GLOBALS['__NMAIL_HTML_PURIFIER'])) {
		ini_set('memory_limit', '512M');

		require_once dirname($G_SYS['PAGE_ROOT']) . '/vendor/ezyang/htmlpurifier/library/HTMLPurifier.auto.php';

		$config = HTMLPurifier_Config::createDefault();
		$config->loadArray(purifier_config());

		$GLOBALS['__NMAIL_HTML_PURIFIER'] = new HTMLPurifier($config);
	}

	$GLOBALS['__NMAIL_HTML_PURIFIER_CLEAN'] = true;

	try {
		if (is_local()) {
			$clean_html = $GLOBALS['__NMAIL_HTML_PURIFIER']->purify($content);
		} else {
			$clean_html = @$GLOBALS['__NMAIL_HTML_PURIFIER']->purify($content);
		}

		// 필터링후 내용 길이가 같을 경우, [HTML 원본 보기] 생략
		if (strlen(trim($content)) === strlen(trim($clean_html))) {
			$GLOBALS['__NMAIL_HTML_PURIFIER_CLEAN'] = false;
		}
	} catch (Exception $e) {
		error("[purifier_clean] HTML purifier 실패 : " . $e->getMessage(), 'SECURITY');

		if (is_local()) {
			throw $e;
		}

		$clean_html = '(메일 본문에서 안전하지 않은 문자열 삭제가 실패하였습니다. 안전한 메일일 경우 화면 상단의 \'HTML 원본 보기\' 를 클릭하시면 됩니다. )';
	}

	// 디버깅
	if (is_local() && $_GET['is_debug']) {
		?>
		<h5 style="color: blue"># purifier_clean (<?= strlen($content) ?> -> <?= strlen($clean_html) ?> , <?= strlen(
				strip_tags($content)
			) ?> -> <?= strlen(strip_tags($clean_html)) ?>)</h5>
		<table width="100%" border="1">
			<tbody>
			<tr>
				<td width="50%" style="vertical-align: top"><?= $content ?></td>
				<td width="50%" style="vertical-align: top"><?= $clean_html ?></td>
			</tr>
			<tr>
				<td style="vertical-align: top"><?= nl2br(e($content)) ?></td>
				<td style="vertical-align: top"><?= nl2br(e($clean_html)) ?></td>
			</tr>
			</tbody>
		</table>
		<?php
	}

	return $clean_html;
}
